package cz.upol.scripting;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javafx.animation.KeyFrame;
import javafx.animation.KeyValue;
import javafx.animation.Timeline;
import javafx.scene.layout.Pane;
import javafx.util.Duration;

/**
 * Reprezentuje graficke zobrazeni herniho sveta v prostredi JavaFX
 */
public class WorldView extends Pane {
	
	/** jednotlive graficke prvky pristupne pres jejich id */
	private Map<String, ElementView> elementViews = new HashMap<>();
	
	/**
	 * Prida do grafickeho zobrazeni novy element
	 */
	public void add(ElementView elementView) {
		this.getChildren().add(elementView);
		elementViews.put(elementView.getElement().getId(), elementView);
	}
	
	/**
	 * Zobrazi zmenu v hernim svete pomoci animace
	 */
	public void singleUpdate() {
		
		KeyValue[] animation;
		List<ElementView> toRemove = null;
		
		// samostatna TimeLine pro animace presunu
		Timeline timeline = new Timeline(30);
		for (ElementView ev : elementViews.values()) {
			boolean disappering = ev.getElement().isDisappering();
			
			if (disappering) {
				animation = new KeyValue[] { new KeyValue(ev.opacityProperty(), 0) };
				toRemove = addLazily(toRemove, ev);
			} else {
				animation = ev.getAnimationValues();
			}
			
			KeyFrame kf = new KeyFrame(Duration.millis(MainApp.TIME_SLICE * 0.95), animation);
			timeline.getKeyFrames().add(kf);
		}
		if (toRemove != null) {
			final List<ElementView> finalToRemove = toRemove; 
			timeline.setOnFinished(e -> removeAll(finalToRemove));
		}
		timeline.play();
	}
	
	private void removeAll(Collection<ElementView> toRemove) {
		for (ElementView rev : toRemove) {
			elementViews.remove(rev.getId());
			this.getChildren().remove(rev);
		}
	}
	
	private static <T> List<T> addLazily(List<T> list, T item) {
		if (list == null) list = new ArrayList<>(4);
		list.add(item);
		return list;
	}

	/**
	 * Odstrani vsechny graficke prvky ze zobrazeni
	 */
	public void clear() {
		elementViews.clear();
		this.getChildren().clear();
	}
}
